/*  Webpen : an applet to draw graffitis on a video stream in pd
 *  skeleton inspired by java applet by Damien Henry : http://dh7.free.fr
 *  Yves Degoyon 2003 
 */

import java.awt.event.*;
import java.awt.*;
import java.lang.Math;
import java.io.*;
import java.net.*;

public class Webpen extends java.applet.Applet implements MouseListener, MouseMotionListener {
  int             mx, my, ms;
  int             red, green, blue;
  int             size;
  String          msgToPd;
  String          statusMsg;
  Socket          toPd = null;
  PrintWriter     out = null;
  BufferedReader  in = null;
  String          pdHost;
  String          pdPort_String;
  Integer         pdPort;
  Graphics        gc;
  Container       optionsContainer;
  Container       imageContainer;
  Container       connectContainer;
  Button          clearButton;
  Button          fillButton;
  Scrollbar       widthScrollbar;
  Label           widthLabel;
  Scrollbar       redScrollbar;
  Label           redLabel;
  Scrollbar       greenScrollbar;
  Label           greenLabel;
  Scrollbar       blueScrollbar;
  Label           blueLabel;
  Button          eraseButton;
  int             eraseMode=0;
  Label           statusLabel;
  Label           hostLabel;
  TextField       hostText;
  Label           portLabel;
  TextField       portText;
  Button          connectButton;
  Label           connectStatusLabel;
  Image           darea;
  int             awidth=500, aheight=400;
  int             nbpoints=0;
  int             connected=0;

  public void init() {

    red = green = blue = 255;
    size = 3;
    darea = createImage(awidth, aheight);
    gc = darea.getGraphics();
    gc.setColor(Color.black);
    gc.fillRect(0,0,awidth, aheight);

    addMouseListener(this);
    addMouseMotionListener(this);

    this.resize( awidth+150, aheight+100 );
    this.setLayout(new BorderLayout());
    optionsContainer = new Container();
    imageContainer = new Container();
    connectContainer = new Container();

    imageContainer.setSize(awidth, aheight);
    imageContainer.setVisible(true);
    this.add( imageContainer, BorderLayout.CENTER );

    optionsContainer.setLayout(new GridLayout(11,1));
    //fillButton = new Button("Fill");
    //fillButton.addActionListener(new FillListener(this));
    //optionsContainer.add(fillButton);
    widthScrollbar = new Scrollbar( Scrollbar.HORIZONTAL, 3, 1, 0, 51);
    widthScrollbar.setBlockIncrement(1);    
    widthScrollbar.addAdjustmentListener(new WidthListener(this));
    optionsContainer.add(widthScrollbar);
    widthLabel = new Label("0     Width     50");
    optionsContainer.add(widthLabel);
    redScrollbar = new Scrollbar( Scrollbar.HORIZONTAL, 255, 1, 0, 256);
    redScrollbar.setBlockIncrement(1);    
    redScrollbar.addAdjustmentListener(new RedListener(this));
    optionsContainer.add(redScrollbar);
    redLabel =   new Label("0     Red      255");
    optionsContainer.add(redLabel);
    greenScrollbar = new Scrollbar( Scrollbar.HORIZONTAL, 255, 1, 0, 256);
    greenScrollbar.setBlockIncrement(1);    
    greenScrollbar.addAdjustmentListener(new GreenListener(this));
    optionsContainer.add(greenScrollbar);
    greenLabel = new Label("0     Green    255");
    optionsContainer.add(greenLabel);
    blueScrollbar = new Scrollbar( Scrollbar.HORIZONTAL, 255, 1, 0, 256);
    blueScrollbar.setBlockIncrement(1);    
    blueScrollbar.addAdjustmentListener(new BlueListener(this));
    optionsContainer.add(blueScrollbar);
    blueLabel =  new Label("0     Blue     255");
    optionsContainer.add(blueLabel);
    clearButton = new Button("Clear");
    clearButton.addActionListener(new ClearListener(this));
    optionsContainer.add(clearButton);
    eraseButton = new Button("Erase Mode : Off");
    eraseButton.addActionListener(new EraseListener(this));
    optionsContainer.add(eraseButton);
    statusLabel =  new Label("   Status   ");
    optionsContainer.add(statusLabel);
    optionsContainer.setSize(150, aheight);
    optionsContainer.setVisible(true);
    this.add( optionsContainer, BorderLayout.EAST );

    connectContainer.setLayout(new FlowLayout());
    hostLabel =  new Label("Host :");
    connectContainer.add(hostLabel);
    hostText =  new TextField("", 30);
    connectContainer.add(hostText);
    portLabel =  new Label("Port : ");
    connectContainer.add(portLabel);
    portText =  new TextField("", 4);
    connectContainer.add(portText);
    connectButton = new Button("Connect");
    connectButton.addActionListener(new ConnectListener(this));
    connectContainer.add(connectButton);
    connectStatusLabel =  new Label(">> Disconnected ");
    connectContainer.add(connectStatusLabel);
    connectContainer.setSize(awidth, 100);
    connectContainer.setVisible(true);
    this.add( connectContainer, BorderLayout.SOUTH );
  }

  public void stop() {  } ;

  public void connect( String pdHost, int pdPort)
  {
    try {
      toPd = new Socket(pdHost, pdPort);
      out = new PrintWriter(toPd.getOutputStream(), true);
      in = new BufferedReader(new InputStreamReader(toPd.getInputStream()));
      connected = 1;
    } catch (UnknownHostException e) {
      System.err.println("Unknown host : standalone mode");
      out = null;
      connected = 0;
      return;
    } catch (Exception e) {
      System.err.println("Could not connect to the pd host : standalone mode");
      out = null;
      connected = 0;
      return;
    }
    msgToPd = "Clear ;";
    if (out!=null) out.println(msgToPd);
    msgToPd = "Rgb " + String.valueOf(red) + 
	         " " + String.valueOf(green) +
		 " " + String.valueOf(blue) + ";";
    if (out!=null) out.println(msgToPd);
    msgToPd = "Width " + String.valueOf(size/2) + ";";
    if (out!=null) out.println(msgToPd);
  }

  public void destroy() {
    removeMouseListener(this);
    removeMouseMotionListener(this);
    try {
     if (out!=null) out.close();
     in.close();
     toPd.close();
    } catch (IOException e) {
      System.err.println("IOException while closing socket");
      System.exit(1);
    }
  }

  public void paint(Graphics g) {
    statusMsg = "X: " + String.valueOf(mx) + " Y: " + String.valueOf(my)  ;
    statusLabel.setText( statusMsg );

    g.drawImage(darea, 0, 0, this);

    if ( connected == 0 )
    {
       connectStatusLabel.setText(">> Disconnected");
    }
    else
    {
       connectStatusLabel.setText(">> Connected");
    }
  }

  /*
   * Mouse methods
   */
  public void mouseDragged(MouseEvent e) {
    mx = e.getX();
    my = e.getY();
    ms=10;

    if ( ( mx > awidth ) || ( my > aheight ) )
    {
      return;
    }
    msgToPd = "X " + String.valueOf((float)mx/(float)awidth) + " ; Y " + 
	             String.valueOf((float)my/(float)aheight)  ;
    if (ms == 10) {
      msgToPd = msgToPd + " ; Click 1;" ;
    }
    else 
    {
      msgToPd = msgToPd + " ; Click 0 ;" ;
    }

    if ( eraseMode == 1 )
    {
      gc.setColor( Color.black );
    }
    else
    {
      gc.setColor( new Color( red, green, blue ) );
    }
    gc.drawOval(mx, my, 2*size, 2*size);
    gc.fillOval(mx, my, 2*size, 2*size);
    if (out!=null) out.println(msgToPd)  ;
    nbpoints = ( nbpoints + 1 ) % 100;
    if ( nbpoints % 5 == 0 )
    {
      repaint();
    } 
  }

  public void mouseMoved(MouseEvent e) {
    mx = e.getX();
    my = e.getY();
    ms=20  ;
    // repaint()  ;
  }

  public void mousePressed(MouseEvent e) {
    mx = e.getX();
    my = e.getY();
    ms=10;
    requestFocus();
    repaint()  ;
  }

  public void mouseReleased(MouseEvent e) {
    mx = e.getX();
    my = e.getY();
    ms=20 ;
    repaint();
  }

  public void mouseEntered(MouseEvent e) {
    repaint();
  }

  public void mouseExited(MouseEvent e) {
    // repaint();
  }

  public void mouseClicked(MouseEvent e) {
    repaint()  ;
  }

  public void clear() {
    gc.setColor(Color.black);
    gc.fillRect(0,0,awidth, aheight);
    repaint();
  }

  /**
   * Focus methods
   */
  public void keyDown(int key) {
	requestFocus();
  }

  public String getAppletInfo() {
        return "Title: Webpen\nAuthor: Yves Degoyon";
  }

  private class ClearListener implements ActionListener
  {
    Webpen target;
    public ClearListener (Webpen app) 
    {
       this.target=app;
    }
    public void actionPerformed(ActionEvent e)      
    {
       System.out.println( "Click event" );	
       this.target.clear();
       msgToPd = "Clear ;";
       if (this.target.connected == 1) this.target.out.println(msgToPd);
    }
  }

  private class FillListener implements ActionListener
  {
    Webpen target;
    public FillListener (Webpen app) 
    {
       this.target=app;
    }
    public void actionPerformed(ActionEvent e)      
    {
       msgToPd = "Fill " + String.valueOf((float)target.mx/(float)awidth) + " " + 
	                   String.valueOf((float)target.my/(float)aheight) + ";";
       if (this.target.connected == 1) this.target.out.println(msgToPd);
    }
  }

  private class WidthListener implements AdjustmentListener
  {
    Webpen target;
    public WidthListener (Webpen app) 
    {
       this.target=app;
    }
    public void adjustmentValueChanged(AdjustmentEvent e)      
    {
       this.target.size = this.target.widthScrollbar.getValue();
       msgToPd = "Width " + String.valueOf(this.target.size/2) + ";";
       if (this.target.connected == 1) this.target.out.println(msgToPd);
    }
  }

  private class RedListener implements AdjustmentListener
  {
    Webpen target;
    public RedListener (Webpen app) 
    {
       this.target=app;
    }
    public void adjustmentValueChanged(AdjustmentEvent e)      
    {
       this.target.red = this.target.redScrollbar.getValue();
       msgToPd = "Rgb " + String.valueOf(this.target.red) + 
	         " " + String.valueOf(this.target.green) +
		 " " + String.valueOf(this.target.blue) + ";";
       if (this.target.connected == 1) this.target.out.println(msgToPd);
    }
  }

  private class GreenListener implements AdjustmentListener
  {
    Webpen target;
    public GreenListener (Webpen app) 
    {
       this.target=app;
    }
    public void adjustmentValueChanged(AdjustmentEvent e)      
    {
       this.target.green = this.target.greenScrollbar.getValue();
       msgToPd = "Rgb " + String.valueOf(this.target.red) + 
	         " " + String.valueOf(this.target.green) +
		 " " + String.valueOf(this.target.blue) + ";";
       if (this.target.connected == 1) this.target.out.println(msgToPd);
    }
  }

  private class BlueListener implements AdjustmentListener
  {
    Webpen target;
    public BlueListener (Webpen app) 
    {
       this.target=app;
    }
    public void adjustmentValueChanged(AdjustmentEvent e)      
    {
       this.target.blue = this.target.blueScrollbar.getValue();
       msgToPd = "Rgb " + String.valueOf(this.target.red) + 
	         " " + String.valueOf(this.target.green) +
		 " " + String.valueOf(this.target.blue) + ";";
       if (this.target.connected == 1) this.target.out.println(msgToPd);
    }
  }

  private class EraseListener implements ActionListener
  {
    Webpen target;
    public EraseListener (Webpen app) 
    {
       this.target=app;
    }
    public void actionPerformed(ActionEvent e)      
    {
       if ( this.target.eraseMode == 1) 
       {
	 this.target.eraseMode = 0;
         msgToPd = "Mode 0;";
         if (this.target.connected == 1) this.target.out.println(msgToPd);
	 this.target.eraseButton.setLabel("Erase Mode : Off");
       }
       else
       {
	 this.target.eraseMode = 1;
         msgToPd = "Mode 1;";
         if (this.target.connected == 1) this.target.out.println(msgToPd);
	 this.target.eraseButton.setLabel("Erase Mode : On");
       }
    }
  }

  private class ConnectListener implements ActionListener
  {
    Webpen target;
    public ConnectListener (Webpen app) 
    {
       this.target=app;
    }
    public void actionPerformed(ActionEvent e)      
    {
       String host = this.target.hostText.getText();
       int port = new Integer( this.target.portText.getText() ).intValue();

       target.connect( host, port );
       target.clear();
       repaint();
    }
  }
}
